/****************************************************************

  Author: Antonio Cisternino

  Version: 1.5.24

  Description: This file contains the parser code. See the 
               parserdefs.h file to hook the parser events.

 ****************************************************************/

#include <ctype.h>
#include <stdio.h>
#include "parser.h"
#include "parserdefs.h"

rc_result_t analyze(char *p) {
  char *input = p;
  rc_obj_t objname;
  int i, time, num;
  char teamname[MAX_TEAMNAME_LEN], labels[3];
  float dist, dir, distchg, dirchg, bodydir, headdir, speed;
  rc_state_t state;
  char msg[MAX_MSG_LEN];
  rc_view_quality_t viewQuality;
  rc_view_width_t viewWidth;
  rc_error_t error_type;
  int stamina, effort, kickCount, dashCount, turnCount, 
      sayCount, turnNeckCount;

  switch (input[3]) {
  case 'i': /* init */
    input += 6;

    labels[0] = *input;
    input += 2;

    READ_INT(num);
    ++input;

    CHECK_STATE(labels[0]);

    ON_INIT(labels[0], num, state);
    break;
  case 'r': /* error */
    input += 7;

    if (*input == 'u') { /* unknown_command */
      error_type = E_UNKNOWN_COMMAND;
    } else { /* illigal_command_form */
      error_type = E_ILLIGAL_COMMAND_FORM;
    }

    ON_ERROR(error_type);
    break;
  case 'e': /* see */
    input += 5;

    READ_INT(time);

    ON_SEE(time);

    while (*input != ')') { /* Read the object info */
      labels[0] = labels[1] = labels[2] = '\0';
      dist = dir = distchg = dirchg = bodydir = headdir = INVALID_VALUE;

      input += 3; /* eat " ((" */

      switch (*input) {
      case 'p': /* player */
	objname = OBJ_PLAYER;
	input += 6;

	num = INVALID_VALUE;
	teamname[0] = '\0';

	if (*input++ == ')')
	  break;

	i = 0;
	while (!isspace(*input) && (*input != ')')) ++i, ++input;

	strncpy(teamname, input - i, i);

	teamname[i] = '\0';

	if (*input++ == ')')
	  break;

	READ_INT(num);

        input++;

	break;
      case 'g': /* goal */
	objname = OBJ_GOAL;
	labels[0] = input[5];

	input += 7;
	break;
      case 'b': /* ball */
	objname = OBJ_BALL;
	input += 5;
	break;
      case 'f': /* flag */
	objname = OBJ_FLAG;
	input += 5;

	labels[0] = *input++;

	if (*input++ == ')')
	  break;

	labels[1] = *input++;

	if (*input++ == ')')
	  break;

	labels[2] = *input++;

	if (*input == ')')
	  ++input;
	else
	  input += 2;
	break;
      case 'l': /* line */
	objname = OBJ_LINE;
	labels[0] = input[5];

	input += 7;
	break;
      }

      if (*input != ')') {
	++input;
      
	READ_NUM(dist);

	if (*input != ')') {
	  ++input;

	  READ_NUM(dir);
	  
	  if (*input != ')') {
	    ++input;
	    
	    READ_NUM(distchg);
	    
	    if (*input != ')') {
	      ++input;

	      READ_NUM(dirchg);
	      
	      if (*input != ')') {
		++input;

		READ_NUM(bodydir);

		if (*input != ')') {
		  ++input;

		  READ_NUM(headdir);
		}
	      }
	    }
	  }
	}
      }

      ++input;

      switch (objname) {
      case OBJ_PLAYER:
	ON_SEE_PLAYER(teamname, num, dist, dir, distchg, dirchg, bodydir, headdir);
	break;
      case OBJ_GOAL:
	ON_SEE_GOAL(labels[0], dist, dir);
	break;
      case OBJ_BALL:
	ON_SEE_BALL(dist, dir, distchg, dirchg);
	break;
      case OBJ_FLAG:
	ON_SEE_FLAG(labels[0], labels[1], labels[2], dist, dir, distchg, dirchg);
	break;
      case OBJ_LINE:
	ON_SEE_LINE(labels[0], dist, dir);
	break;
      }
    }
    break;
  case 'a': /* hear */
    input += 6;

    READ_INT(time);

    ++input; /* white space */

    switch (*input) {
    case 's':
      dir = SELF_DIR;
      input += 5;
      break;
    case 'r':
      dir = REFEREE_DIR;
      input += 8;
      break;
    default:
      READ_NUM(dir);
      ++input;
    }

    if (dir == REFEREE_DIR) {
      labels[0] = '\0';
      num = -1;

      CHECK_STATE(labels[0]);

      ON_REFEREE(time, state, labels[0], num);
    } else {
      strcpy(msg, input);
      msg[strlen(msg) - 1] = '\0';

      ON_HEAR(time, dir, msg);
    }
    break;
  case 'n': /* sense_body */
    input += 12;

    READ_INT(time);

    ++input; /* white space */

    /* view_mode */
    input += 11;

    switch (*input) {
    case 'h':
      viewQuality = VQ_HIGH;
      input += 5;
      break;
    case 'l':
      viewQuality = VQ_LOW;
      input += 4;
      break;
    }

    switch (input[1]) {
    case 'a':
      viewWidth = VW_NARROW;
      input += 7;
      break;
    case 'o':
      viewWidth = VW_NORMAL;
      input += 7;
      break;
    case 'i':
      viewWidth = VW_WIDE;
      input += 5;
      break;
    }

    /* stamina */
    input += 10;

    READ_INT(stamina);

    ++input;

    READ_INT(effort);

    ++input;

    /* speed */
    input += 8;

    READ_NUM(speed);

    ++input;

    /* head_angle */
    input += 13;
    
    READ_NUM(headdir);

    ++input;

    /* kick */
    input += 7;

    READ_INT(kickCount);

    ++input;

    /* dash */
    input += 7;

    READ_INT(dashCount);

    ++input;

    /* turn */
    input += 7;

    READ_INT(turnCount);

    ++input;

    /* say */
    input += 6;

    READ_INT(sayCount);

    ++input;

    /* turn_neck */
    input += 12;

    READ_INT(turnNeckCount);

    ++input;

    ON_SENSE_BODY(time, viewQuality, viewWidth, stamina, effort, speed, headdir, kickCount, dashCount, turnCount, sayCount, turnNeckCount);
    break;
  }

  return OK;
}

